#include "utils.h"
#include "assetmeta.h"
#include "processing.h"

#include <string.h>

typedef struct {
    char path_scratch[1024];
    unsigned int path_end;
} vy_discovery_data;

static vy_result DirectoryHandler(const char *name, vyIterateDirElementType type, void *user) {
    vy_discovery_data *data = user;

    size_t name_len = strlen(name);

    if (type == VY_DIR_ELEMENT_TYPE_FILE) {
        /* Don't parse .as files */
        if (name_len >= 3) {
            if (memcmp(&name[name_len - 3], ".as", 3) == 0) {
                /* Skip */
                return VY_SUCCESS;
            }
        }

        /* Check if we know that file */
        data->path_scratch[data->path_end] = '/';
        memcpy(data->path_scratch + data->path_end + 1, name, name_len);
        data->path_scratch[data->path_end + 1 + name_len] = '\0';

        vy_file_id fid = vyAddFile(data->path_scratch);
        if (vyLookupUID(fid) != VY_INVALID_UID) {
            vy_assetmeta meta;
            if (!vyGetAssetMeta(fid, &meta) || (meta.last_changed >= meta.compiled_ts)) {
                /* The file (may have) changed */
                vy_result res = vyAddFileToProcessingQueue(fid, 0);
                if (res != VY_SUCCESS)
                    return res;
            }
            else {
                /* The file is unchanged, we just need to add it to the output again. */
                vyLog("ASSETC", "File %s is unchanged.", data->path_scratch);
            }
        } else {
            /* Process it */
            vy_result res = vyAddFileToProcessingQueue(fid, 0);
            if (res != VY_SUCCESS)
                return res;
        }
    } else if (type == VY_DIR_ELEMENT_TYPE_DIRECTORY) {
        if (strcmp(name, ".") == 0)
            return VY_SUCCESS;
        if (strcmp(name, "..") == 0)
            return VY_SUCCESS;

        unsigned int path_end_before = data->path_end;
        data->path_scratch[data->path_end++] = '/';
        memcpy(data->path_scratch + data->path_end, name, name_len);
        data->path_scratch[data->path_end + name_len] = '\0';
        data->path_end += name_len;

        vy_result res = vyIterateDirectory(data->path_scratch, user, DirectoryHandler);
        if (res != VY_SUCCESS)
            return res;

        data->path_end = path_end_before;
    }
    return VY_SUCCESS;
}

void vyDiscoverAssets(void) {
    vy_discovery_data data;
    data.path_scratch[0] = '.';
    data.path_scratch[1] = '\0';
    data.path_end        = 1;

    vyIterateDirectory(".", &data, DirectoryHandler);
}